﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using Microsoft.SqlServer.Server;
using System.Collections;

public partial class UserDefinedFunctions
{
    [Microsoft.SqlServer.Server.SqlFunction(
        DataAccess = DataAccessKind.Read, 
        FillRowMethodName = "get_active_products_fill", 
        TableDefinition = "ProductID INT, StartDate DATE, EndDate DATE")]
    public static IEnumerator get_active_products(SqlInt32 ActiveInterval)
    {
        //Not using using, because the connection will be passed around
        //see below
        SqlConnection conn = connection_builder.get_loopback_connection();

        try
        {
            SqlCommand comm = new SqlCommand();
            comm.Connection = conn;
            comm.CommandText = @"
	            SELECT DISTINCT
	                ProductID,
	                TransactionDate
                FROM AdventureWorks.Production.TransactionHistory
                ORDER BY
	                ProductID,
	                TransactionDate;";

            conn.Open();

            //using CommandBehavior.CloseConnection so that
            //the connection can be closed by the enumerator
            return new active_products_enumerator(
                comm.ExecuteReader(CommandBehavior.CloseConnection),
                ActiveInterval.Value);
        }
        catch
        {
            conn.Dispose();
            throw;
        }
    }

    public static void get_active_products_fill(
        object o, 
        out SqlInt32 ProductID, 
        out DateTime StartDate, 
        out DateTime EndDate)
    {
        var r = (active_products_enumerator.results)o;
        ProductID = r.ProductID;
        StartDate = r.StartDate;
        EndDate = r.EndDate;
    }

    private class active_products_enumerator : IEnumerator
    {
        public active_products_enumerator(
            SqlDataReader r,
            int activeInterval)
        {
            this.r = r;
            this.activeInterval = activeInterval;
        }

        private results current_results = null;
        private SqlDataReader r;
        private int activeInterval;

        private DateTime StartDate = new DateTime(1900, 1, 1);

        private SqlInt32 ProductID = 0;
        private SqlInt32 new_ProductID = 0;

        private DateTime period_plus_interval = new DateTime(1900, 1, 1);
        private DateTime new_period_date = new DateTime(1900, 1, 1);

        private readonly DateTime now = DateTime.Now;

        #region IEnumerator Members

        public object Current
        {
            get { return (this.current_results); }
        }

        public bool MoveNext()
        {
            try
            {
                current_results = null;

                while (r.Read())
                {
                    new_ProductID = r.GetSqlInt32(0);
                    new_period_date = r.GetDateTime(1);

                    if (new_ProductID != ProductID)
                    {
                        if (ProductID != 0)
                        {
                            current_results = new results(
                                ProductID,
                                StartDate,
                                period_plus_interval);
                        }

                        ProductID = new_ProductID;
                        StartDate = new_period_date;
                        period_plus_interval = new_period_date.AddDays(activeInterval);

                        if (current_results != null)
                            return (true);
                    }
                    else
                    {
                        if (period_plus_interval < new_period_date)
                        {
                            current_results = new results(
                                ProductID,
                                StartDate,
                                period_plus_interval);

                            StartDate = new_period_date;
                        }

                        period_plus_interval = new_period_date.AddDays(activeInterval);

                        if (current_results != null)
                            return (true);
                    }
                }

                //return the last row of data
                if (ProductID != 0)
                {
                    current_results = new results(
                        ProductID,
                        StartDate,
                        period_plus_interval);

                    //set this to 0 so we don't return another row
                    ProductID = 0;
                }

                if (current_results != null)
                    return (true);
                else
                {
                    r.Dispose();
                    return (false);
                }
            }
            catch
            {
                r.Dispose();
                throw;
            }
        }

        public void Reset()
        {
            throw new NotImplementedException();
        }

        #endregion

        public class results
        {
            public results(SqlInt32 ProductID, DateTime StartDate, DateTime EndDate)
            {
                this.ProductID = ProductID;
                this.StartDate = StartDate;
                this.EndDate = EndDate;
            }

            public readonly SqlInt32 ProductID;
            public readonly DateTime StartDate;
            public readonly DateTime EndDate;
        }
    }

};

